@@ -2,10 +2,11 @@ package DBIx::Class::Schema::Loader::RelBuilder;
 
 use strict;
 use warnings;
+use Class::C3;
 use Carp::Clan qw/^DBIx::Class/;
 use Lingua::EN::Inflect::Number ();
 
-our $VERSION = '0.04006';
+our $VERSION = '0.05000';
 
 =head1 NAME
 
@@ -24,9 +25,9 @@ is module is not (yet) for external use.
 
 =head2 new
 
-Arguments: schema  inflect_plural, inflect_singular
+Arguments: schema_class (scalar), inflect_plural, inflect_singular
 
-C<$schema> should be a schema instance, where the source
+C<$schema_class> should be a schema class name, where the source
 classes have already been set up and registered.  Column info, primary
 key, and unique constraints will be drawn from this schema for all
 of the existing source monikers.
@@ -75,17 +76,23 @@ arguments, like so:
 =cut
 
 sub new {
-    my ( $class, $schema, $inflect_pl, $inflect_singular ) = @_;
+
+    my ( $class, $schema, $inflect_pl, $inflect_singular, $rel_attrs ) = @_;
 
     my $self = {
         schema => $schema,
         inflect_plural => $inflect_pl,
         inflect_singular => $inflect_singular,
+        relationship_attrs => $rel_attrs,
     };
 
-    bless $self => $class;
+    # validate the relationship_attrs arg
+    if( defined $self->{relationship_attrs} ) {
+	ref($self->{relationship_attrs}) eq 'HASH'
+	    or croak "relationship_attrs must be a hashref";
+    }
 
-    $self;
+    return bless $self => $class;
 }
 
 
@@ -121,8 +128,86 @@ sub _inflect_singular {
     return Lingua::EN::Inflect::Number::to_S($relname);
 }
 
+# accessor for options to be passed to each generated relationship
+# type.  take single argument, the relationship type name, and returns
+# either a hashref (if some options are set), or nothing
+sub _relationship_attrs {
+    my ( $self, $reltype ) = @_;
+    my $r = $self->{relationship_attrs};
+    return unless $r && ( $r->{all} || $r->{$reltype} );
+
+    my %composite = %{ $r->{all} || {} };
+    if( my $specific = $r->{$reltype} ) {
+	while( my ($k,$v) = each %$specific ) {
+	    $composite{$k} = $v;
+	}
+    }
+    return \%composite;
+}
+
+sub _array_eq {
+    my ($a, $b) = @_;
+
+    return unless @$a == @$b;
+
+    for (my $i = 0; $i < @$a; $i++) {
+        return unless $a->[$i] eq $b->[$i];
+    }
+    return 1;
+}
+
+sub _uniq_fk_rel {
+    my ($self, $local_moniker, $local_relname, $local_cols, $uniqs) = @_;
+
+    my $remote_method = 'has_many';
+
+    # If the local columns have a UNIQUE constraint, this is a one-to-one rel
+    my $local_source = $self->{schema}->source($local_moniker);
+    if (_array_eq([ $local_source->primary_columns ], $local_cols) ||
+            grep { _array_eq($_->[1], $local_cols) } @$uniqs) {
+        $remote_method = 'might_have';
+        $local_relname = $self->_inflect_singular($local_relname);
+    }
+
+    return ($remote_method, $local_relname);
+}
+
+sub _remote_attrs {
+	my ($self, $local_moniker, $local_cols) = @_;
+
+	# get our base set of attrs from _relationship_attrs, if present
+	my $attrs = $self->_relationship_attrs('belongs_to') || {};
+
+	# If the referring column is nullable, make 'belongs_to' an
+	# outer join, unless explicitly set by relationship_attrs
+	my $nullable = grep { $self->{schema}->source($local_moniker)->column_info($_)->{is_nullable} }
+		@$local_cols;
+	$attrs->{join_type} = 'LEFT'
+	    if $nullable && !defined $attrs->{join_type};
+
+	return $attrs;
+}
+
+sub _remote_relname {
+    my ($self, $remote_table, $cond) = @_;
+
+    my $remote_relname;
+    # for single-column case, set the remote relname to the column
+    # name, to make filter accessors work, but strip trailing _id
+    if(scalar keys %{$cond} == 1) {
+        my ($col) = values %{$cond};
+        $col =~ s/_id$//;
+        $remote_relname = $self->_inflect_singular($col);
+    }
+    else {
+        $remote_relname = $self->_inflect_singular(lc $remote_table);
+    }
+
+    return $remote_relname;
+}
+
 sub generate_code {
-    my ($self, $local_moniker, $rels) = @_;
+    my ($self, $local_moniker, $rels, $uniqs) = @_;
 
     my $all_code = {};
 
@@ -156,17 +241,7 @@ sub generate_code {
         }
 
         my $local_relname;
-        my $remote_relname;
-
-        # for single-column case, set the remote relname to the column
-        # name, to make filter accessors work
-        if(scalar keys %cond == 1) {
-            my ($col) = keys %cond;
-            $remote_relname = $self->_inflect_singular($cond{$col});
-        }
-        else {
-            $remote_relname = $self->_inflect_singular(lc $remote_table);
-        }
+        my $remote_relname = $self->_remote_relname($remote_table, \%cond);
 
         # If more than one rel between this pair of tables, use the local
         # col names to distinguish
@@ -187,28 +262,43 @@ sub generate_code {
             delete $rev_cond{$_};
         }
 
-        push(@{$all_code->{$local_class}{stmts}},
+        my ($remote_method);
+
+        ($remote_method, $local_relname) = $self->_uniq_fk_rel($local_moniker, $local_relname, $local_cols, $uniqs);
+
+        push(@{$all_code->{$local_class}},
             { method => 'belongs_to',
               args => [ $remote_relname,
                         $remote_class,
                         \%cond,
+                        $self->_remote_attrs($local_moniker, $local_cols),
               ],
             }
         );
-        $all_code->{$local_class}{moniker} = $local_moniker;
 
-        push(@{$all_code->{$remote_class}{stmts}},
-            { method => 'has_many',
+        push(@{$all_code->{$remote_class}},
+            { method => $remote_method,
               args => [ $local_relname,
                         $local_class,
                         \%rev_cond,
+			$self->_relationship_attrs($remote_method),
               ],
             }
         );
-        $all_code->{$remote_class}{moniker} = $remote_moniker;
     }
 
     return $all_code;
 }
 
+=head1 AUTHOR
+
+See L<DBIx::Class::Schema::Loader/AUTHOR> and L<DBIx::Class::Schema::Loader/CONTRIBUTORS>.
+
+=head1 LICENSE
+
+This library is free software; you can redistribute it and/or modify it under
+the same terms as Perl itself.
+
+=cut
+
 1;
